@@ -14,30 +14,38 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 
-use 5.6.0;
+use 5.006_000;
 use strict;
 use Cwd;
 use File::Basename;
-use POSIX qw{tmpnam};
+use File::Temp qw/ tempdir /;
 
 # Predeclare functions
 sub process_debc($$);
 sub mktmpdirs();
 sub fatal(@);
 
-(my $progname = $0) =~ s|.*/||;
+my $progname = basename($0);
+my $modified_conf_msg;
 
-my $usage = <<"EOF";
+sub usage {
+    print <<"EOF";
 Usage: $progname [option] ... deb1 deb2
    or: $progname [option] ... changes1 changes2
    or: $progname [option] ... dsc1 dsc2
    or: $progname [option] ... --from deb1a deb1b ... --to deb2a deb2b ...
 Valid options are:
+    --no-conf, --noconf
+                          Don\'t read devscripts config files;
+                          must be the first option given
    --help, -h             Display this message
    --version, -v          Display version and copyright info
    --move FROM TO,        The prefix FROM in first packages has
      -m FROM TO             been renamed TO in the new packages
                             (multiple permitted)
+   --move-regex FROM TO,  The prefix FROM in first packages has
+                            been renamed TO in the new packages
+                            (multiple permitted), using regexp substitution
    --dirs, -d             Note changes in directories as well as files
    --nodirs               Do not note changes in directories (default)
    --nocontrol            Skip comparing control files when comparing
@@ -53,7 +61,11 @@ Valid options are:
    --renamed FROM TO      The package formerly called FROM has been
                             renamed TO; only of interest with --show-moved
                             (multiple permitted)
+
+Default settings modified by devscripts configuration files:
+$modified_conf_msg
 EOF
+}
 
 my $version = <<"EOF";
 This is $progname, from the Debian devscripts package, version ###VERSION###
@@ -65,64 +77,101 @@ You are free to redistribute this code under the terms of the
 GNU General Public License, version 2 ONLY.
 EOF
 
-##
-## handle command-line options
-##
-if (@ARGV and $ARGV[0] =~ /^(--help|-h)$/) { print $usage; exit 0; }
-if (@ARGV and $ARGV[0] =~ /^(--version|-v)$/) { print $version; exit 0; }
+# Start by setting default values
 
-# Now start by reading configuration files and then command line
+my $ignore_dirs = 1;
+my $compare_control = 1;
+my $show_moved = 0;
+my $wdiff_opt = '';
+
+# Next, read read configuration files and then command line
 # The next stuff is boilerplate
 
-my @config_files = ('/etc/devscripts.conf', '~/.devscripts');
-my %config_vars = (
-		   'DEBDIFF_DIRS' => 'no',
-		   'DEBDIFF_CONTROL' => 'yes',
-		   'DEBDIFF_SHOW_MOVED' => 'no',
-		   'DEBDIFF_WDIFF_OPT' => '',
-		   );
+if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
+    $modified_conf_msg = "  (no configuration files read)";
+    shift;
+} else {
+    my @config_files = ('/etc/devscripts.conf', '~/.devscripts');
+    my %config_vars = (
+		       'DEBDIFF_DIRS' => 'no',
+		       'DEBDIFF_CONTROL' => 'yes',
+		       'DEBDIFF_SHOW_MOVED' => 'no',
+		       'DEBDIFF_WDIFF_OPT' => '',
+		       );
+    my %config_default = %config_vars;
+    
+    my $shell_cmd;
+    # Set defaults
+    foreach my $var (keys %config_vars) {
+	$shell_cmd .= "$var='$config_vars{$var}';\n";
+    }
+    $shell_cmd .= 'for file in ' . join(" ",@config_files) . "; do\n";
+    $shell_cmd .= '[ -f $file ] && . $file; done;' . "\n";
+    # Read back values
+    foreach my $var (keys %config_vars) { $shell_cmd .= "echo \$$var;\n" }
+    my $shell_out = `/bin/bash -c '$shell_cmd'`;
+    @config_vars{keys %config_vars} = split /\n/, $shell_out, -1;
 
-my $shell_cmd;
-# Set defaults
-foreach my $var (keys %config_vars) {
-    $shell_cmd .= "$var='$config_vars{$var}';\n";
+    # Check validity
+    $config_vars{'DEBDIFF_DIRS'} =~ /^(yes|no)$/
+	or $config_vars{'DEBDIFF_DIRS'}='no';
+    $config_vars{'DEBDIFF_CONTROL'} =~ /^(yes|no)$/
+	or $config_vars{'DEBDIFF_CONTROL'}='yes';
+    $config_vars{'DEBDIFF_SHOW_MOVED'} =~ /^(yes|no)$/
+	or $config_vars{'DEBDIFF_SHOW_MOVED'}='no';
+
+    foreach my $var (sort keys %config_vars) {
+	if ($config_vars{$var} ne $config_default{$var}) {
+	    $modified_conf_msg .= "  $var=$config_vars{$var}\n";
+	}
+    }
+    $modified_conf_msg ||= "  (none)\n";
+    chomp $modified_conf_msg;
+
+    $ignore_dirs = $config_vars{'DEBDIFF_DIRS'} eq 'yes' ? 0 : 1;
+    $compare_control = $config_vars{'DEBDIFF_CONTROL'} eq 'no' ? 0 : 1;
+    $show_moved = $config_vars{'DEBDIFF_SHOW_MOVED'} eq 'yes' ? 1 : 0;
+    $wdiff_opt = $config_vars{'DEBDIFF_WDIFF_OPT'} =~ /^-([plt])$/ ? $1 : '';
 }
-$shell_cmd .= 'for file in ' . join(" ",@config_files) . "; do\n";
-$shell_cmd .= '[ -f $file ] && . $file; done;' . "\n";
-# Read back values
-foreach my $var (keys %config_vars) { $shell_cmd .= "echo \$$var;\n" }
-my $shell_out = `/bin/sh -c '$shell_cmd'`;
-@config_vars{keys %config_vars} = split /\n/, $shell_out, -1;
-
-my $ignore_dirs = $config_vars{'DEBDIFF_DIRS'} eq 'yes' ? 0 : 1;
-my $compare_control = $config_vars{'DEBDIFF_CONTROL'} eq 'no' ? 0 : 1;
-my $show_moved = $config_vars{'DEBDIFF_SHOW_MOVED'} eq 'yes' ? 1 : 0;
-my $wdiff_opt = $config_vars{'DEBDIFF_SHOW_MOVED'} =~ /^-([plt])$/ ? $1 : '';
 
 # Are they a pair of debs, changes or dsc files, or a list of debs?
 my $type = '';
 my @move = ();
 my %renamed = ();
 
-while (@ARGV > 2) {
-    if ($ARGV[0] =~ /^(--move|-m)$/) {
-	fatal $usage unless @ARGV >= 5;
+##
+## handle command-line options
+##
+
+while (@ARGV) {
+    if ($ARGV[0] =~ /^(--help|-h)$/) { usage(); exit 0; }
+    if ($ARGV[0] =~ /^(--version|-v)$/) { print $version; exit 0; }
+    if ($ARGV[0] =~ /^(--move(-regex)?|-m)$/) {
+	fatal "Malformed command-line options; run $progname --help for more info"
+	    unless @ARGV >= 3;
+
+	my $regex = $ARGV[0] eq '--move-regex' ? 1 : 0;
 	shift @ARGV;
 
 	# Ensure from and to values all begin with a slash
-	# This is because the format of the tar archive has changed between
-	# slink and potato, and we must recognise both forms:
-	# slink: usr/lib/filename
-	# potato: ./usr/lib/filename
+	# From potato onward, dpkg -c gives filenames such as:
+	#  ./usr/lib/filename
 	my $from = shift;
 	my $to   = shift;
-	$from =~ s,^\./,/,; $from =~ s,^([^/]),/$1,;
-	$to   =~ s,^\./,/,; $to   =~ s,^([^/]),/$1,;
+	$from =~ s%^\./%/%;
+	$to   =~ s%^\./%/%;
 
-	push @move, [$from, $to];
+	if ($regex) {
+	    # quote ':' in the from and to patterns;
+	    # used later as a pattern delimiter
+	    $from =~ s/:/\\:/g;
+	    $to =~ s/:/\\:/g;
+	}
+	push @move, [$regex, $from, $to];
     }
     elsif ($ARGV[0] eq '--renamed') {
-	fatal $usage unless @ARGV >= 5;
+	fatal "Malformed command-line options; run $progname --help for more info"
+	    unless @ARGV >= 3;
 	shift @ARGV;
 
 	my $from = shift;
@@ -137,13 +186,24 @@ while (@ARGV > 2) {
     elsif ($ARGV[0] eq '--control') { $compare_control = 1; shift; }
     elsif ($ARGV[0] eq '--from') { $type = 'debs'; last; }
     elsif ($ARGV[0] =~ /^--w([plt])$/) { $wdiff_opt = "-$1"; shift; }
+    elsif ($ARGV[0] =~ /^--no-?conf$/) {
+	fatal "--no-conf is only acceptable as the first command-line option!";
+    }
 
-    else { fatal $usage; }
+    # Not a recognised option
+    elsif ($ARGV[0] =~ /^-/) {
+	fatal "Unrecognised command-line option $ARGV[0]; run $progname --help for more info";
+    }
+    else {
+	# End of command line options
+	last;
+    }
 }
 
 if (! $type) {
     # we need 2 deb files or changes files to compare
-    fatal $usage unless @ARGV == 2;
+    fatal "Need exactly two deb files or changes files to compare"
+	unless @ARGV == 2;
 
     foreach my $i (0,1) {
 	fatal "Can't read file: $ARGV[$i]" unless -r $ARGV[$i];
@@ -157,7 +217,7 @@ if (! $type) {
 	fatal "Could not recognise files; the names should end .deb, .changes or .dsc";
     }
     if ($ARGV[1] !~ /\.$type$/) {
-	unless ($type == 'deb' and `file $ARGV[0]` =~ /Debian/) {
+	unless ($type eq 'deb' and `file $ARGV[0]` =~ /Debian/) {
 	    fatal "The two filenames must have the same suffix, either .deb, .changes or .dsc";
 	}
     }
@@ -295,7 +355,7 @@ elsif ($type eq 'dsc') {
     }
 
     # Do we have interdiff?
-    system("interdiff --version >/dev/null 2>&1");
+    system("(interdiff --version) >/dev/null 2>&1");
     my $use_interdiff = ($?==0) ? 1 : 0;
 
     if ($origs[1] eq $origs[2] and $use_interdiff) {
@@ -326,7 +386,6 @@ elsif ($type eq 'dsc') {
 	    ${"sdir$i"} = $1;
 	}
 	system ("diff", "-Nru", "$dir1/$sdir1", "$dir2/$sdir2");
-	system ("rm", "-rf", $dir1, $dir2);
     }
 
     exit 0;
@@ -542,9 +601,11 @@ sub process_debc($$)
     # Do the "move" substitutions in the order received for the first debs
     if ($number == 1) {
 	for my $move (@move) {
-	    my $from = $$move[0];
-	    my $to   = $$move[1];
-	    map { $_ =~ s/^$from/$to/ } @filelist;
+	    my $regex = $$move[0];
+	    my $from  = $$move[1];
+	    my $to    = $$move[2];
+	    map { if ($regex) { eval "\$_ =~ s:^$from:$to:"; }
+	          else { $_ =~ s/^\Q$from\E/$to/; } } @filelist;
 	}
     }
 
@@ -556,15 +617,9 @@ sub mktmpdirs ()
     no strict 'refs';
 
     for my $i (1,2) {
-	my $count=0;
-	do {
-	    $count++;
-	    if ($count==10) {
-		rmdir $dir1 if $i==2;
-		fatal "Couldn't create temp directory";
-	    }
-	    ${"dir$i"}=tmpnam();
-	} until mkdir(${"dir$i"});
+	${"dir$i"}=tempdir( CLEANUP => 1 );
+	fatal "Couldn't create temp directory"
+	    if not defined ${"dir$i"};
     }
 }
 
