@@ -44,6 +44,8 @@
 # Abort if anything goes wrong
 set -e
 
+PROGNAME=`basename $0`
+
 usage () {
     echo \
 "Usage: debrsign [options] [username@]remotehost [changes or dsc]
@@ -59,7 +61,7 @@ usage () {
     --help          Show this message
     --version       Show version and copyright information
   If a changes or dscfile is specified, it is signed, otherwise
-  debian/changes is parsed to find the changes file.  The signing
+  debian/changelog is parsed to find the changes file.  The signing
   is performed on remotehost using ssh and debsign."
 }
 
@@ -75,10 +77,10 @@ GNU General Public License, version 2 or later."
 mustsetvar () {
     if [ "x$2" = x ]
     then
-	echo >&2 "$0: unable to determine $3"
+	echo >&2 "$PROGNAME: unable to determine $3"
 	exit 1
     else
-	# echo "$0: $3 is $2"
+	# echo "$PROGNAME: $3 is $2"
 	eval "$1=\"\$2\""
     fi
 }
@@ -133,7 +135,7 @@ case $# in
 		dsc=`echo $changes | \
 		    perl -pe 's/\.changes$/.dsc/; s/(.*)_(.*)_(.*)\.dsc/\1_\2.dsc/'`
 		;;
-	    *)	echo "$0: Only a .changes or .dsc file is allowed as second argument!" >&2
+	    *)	echo "$PROGNAME: Only a .changes or .dsc file is allowed as second argument!" >&2
 		exit 1 ;;
 	esac
 	;;
@@ -141,7 +143,7 @@ case $# in
     1)	remotehost="$1"
 	# We have to parse debian/changelog to find the current version
 	if [ ! -r debian/changelog ]; then
-	    echo "$0: Must be run from top of source dir or a .changes file given as arg" >&2
+	    echo "$PROGNAME: Must be run from top of source dir or a .changes file given as arg" >&2
 	    exit 1
 	fi
 
@@ -152,11 +154,6 @@ case $# in
 
 	if [ "x$sourceonly" = x ]
 	then
-	    if ! command -v dpkg-architecture > /dev/null 2>&1; then
-		echo "This program depends on dpkg-architecture; your dpkg is far too old" >& 1
-		exit 1
-	    fi
-
 	    mustsetvar arch "`dpkg-architecture -a${targetarch} -t${targetgnusystem} -qDEB_HOST_ARCH`" "build architecture"
 	else
 	    arch=source
@@ -169,7 +166,7 @@ case $# in
 	changes="../$pva.changes"
 	;;
 
-    *)	echo "Usage: $0 [options] [user@]remotehost [.changes or .dsc file]" >&2
+    *)	echo "Usage: $PROGNAME [options] [user@]remotehost [.changes or .dsc file]" >&2
 	exit 1 ;;
 esac
 
