@@ -13,6 +13,7 @@
 #  -a<arch>    Debian architecture
 #  -t<type>    GNU machine type
 #  -r [username@]remotehost  The changes (and dsc) files live on remotehost
+#  --no-conf, --noconf  Don't read configuration files
 #  --help, --version
 
 # Debian GNU/Linux debsign.  Copyright (C) 1999 Julian Gilbey.
@@ -36,6 +37,8 @@
 set -e
 
 PRECIOUS_FILES=0
+PROGNAME=`basename $0`
+MODIFIED_CONF_MSG='Default settings modified by devscripts configuration files:'
 
 # --- Functions
 
@@ -57,11 +60,16 @@ usage () {
     -S              Use changes file made for source-only upload
     -a<arch>        Use changes file made for Debian target architecture <arch>
     -t<target>      Use changes file made for GNU target architecture <target>
+    --no-conf, --noconf
+                    Don't read devscripts config files;
+                    must be the first option given
     --help          Show this message
     --version       Show version and copyright information
   If a changes or dsc file is specified, it and any .dsc files in the
-  changes file are signed, otherwise debian/changes is parsed to find
-  the changes file."
+  changes file are signed, otherwise debian/changelog is parsed to find
+  the changes file.
+
+$MODIFIED_CONF_MSG"
 }
 
 version () {
@@ -75,7 +83,7 @@ GNU General Public License, version 2 or later."
 
 cleanup_tmpdir () {
     if [ "$PRECIOUS_FILES" -gt 0 ]; then
-        echo "$0: aborting with $PRECIOUS_FILES signed files in `pwd`" >&2
+        echo "$PROGNAME: aborting with $PRECIOUS_FILES signed files in `pwd`" >&2
     else
         cd ..; rm -rf debsign.$$
     fi
@@ -84,10 +92,10 @@ cleanup_tmpdir () {
 mustsetvar () {
     if [ "x$2" = x ]
     then
-	echo >&2 "$0: unable to determine $3"
+	echo >&2 "$PROGNAME: unable to determine $3"
 	exit 1
     else
-	# echo "$0: $3 is $2"
+	# echo "$PROGNAME: $3 is $2"
 	eval "$1=\"\$2\""
     fi
 }
@@ -100,19 +108,10 @@ signfile () {
     if [ $signinterface = gpg ]
     then
 	(cat "$1" ; echo "") | \
-	    $signcommand --local-user "$2" --clearsign \
+	    $signcommand --local-user "$2" --clearsign --no-show-policy-url \
 		--armor --textmode --output - - > "$1.asc" || \
 	{ SAVESTAT=$?
-	  [ $SAVESTAT -eq 2 ] && \
-	  echo "$0: GPG error occurred!
-
-If you are using gpg version >= 1.0.3-2 and saw a gpg error message:
-  error loading extension: /usr/lib/gnupg/rsa(ref)
-above, please remove the "load-extension rsa" or "load-extension rsaref"
-line in your ~/.gnupg/options file to ensure the correct functioning of
-this (and possibly other) programs.
-
-Aborting...." >&2
+	  echo "$PROGNAME: gpg error occurred!  Aborting...." >&2
 	  exit $SAVESTAT
 	}
     else
@@ -153,23 +152,59 @@ check_already_signed () {
 
 # --- main script
 
-# For partial security, even though we know it doesn't work :(
-# I guess maintainers will have to be careful, and there's no way around
-# this in a shell script.
-unset IFS
-PATH=/usr/local/bin:/usr/bin:/bin
-umask `perl -e 'printf "%03o\n", umask | 022'`
-
 # Boilerplate: set config variables
-for file in /etc/devscripts.conf ~/.devscripts
-do
-    [ -r $file ] && . $file
-done
+DEFAULT_DEBSIGN_PROGRAM=
+DEFAULT_DEBSIGN_SIGNLIKE=
+DEFAULT_DEBSIGN_MAINT=
+DEFAULT_DEBSIGN_KEYID=
+VARS="DEBSIGN_PROGRAM DEBSIGN_SIGNLIKE DEBSIGN_MAINT DEBSIGN_KEYID"
 
-case "$DEBSIGN_SIGNLIKE" in
-    gpg|pgp) forcesigninterface=$DEBSIGN_SIGNLIKE ;;
-    # otherwise it's ignored
-esac
+if [ "$1" = "--no-conf" -o "$1" = "--noconf" ]; then
+    shift
+    MODIFIED_CONF_MSG="$MODIFIED_CONF_MSG
+  (no configuration files read)"
+
+    # set defaults
+    for var in $VARS; do
+	eval "$var=\$DEFAULT_$var"
+    done
+else
+    # Run in a subshell for protection against accidental errors
+    # in the config files
+    eval $(
+	set +e
+	for var in $VARS; do
+	    eval "$var=\$DEFAULT_$var"
+	done
+
+	for file in /etc/devscripts.conf ~/.devscripts
+	  do
+	  [ -r $file ] && . $file
+	done
+
+	set | egrep '^(DEBSIGN|DEVSCRIPTS)_')
+
+    # check sanity
+    case "$DEBSIGN_SIGNLIKE" in
+	gpg|pgp) ;;
+	*) DEBSIGN_SIGNLIKE= ;;
+    esac
+
+    # set config message
+    MODIFIED_CONF=''
+    for var in $VARS; do
+	eval "if [ \"\$$var\" != \"\$DEFAULT_$var\" ]; then
+	    MODIFIED_CONF_MSG=\"\$MODIFIED_CONF_MSG
+  $var=\$$var\";
+	MODIFIED_CONF=yes;
+	fi"
+    done
+
+    if [ -z "$MODIFIED_CONF" ]; then
+	MODIFIED_CONF_MSG="$MODIFIED_CONF_MSG
+  (none)"
+    fi
+fi
 
 maint="$DEBSIGN_MAINT"
 signkey="$DEBSIGN_KEYID"
@@ -178,7 +213,7 @@ signcommand=''
 if [ -n "$DEBSIGN_PROGRAM" ]; then
     signcommand="$DEBSIGN_PROGRAM"
 else
-    if [ \( -n "$GNUPGHOME" -a -e $GNUPGHOME/secring.gpg \) -o -e $HOME/.gnupg/secring.gpg ] && \
+    if [ \( -n "$GNUPGHOME" -a -e "$GNUPGHOME" \) -o -e $HOME/.gnupg ] && \
 	command -v gpg > /dev/null 2>&1; then
 	signcommand=gpg
     elif command -v pgp > /dev/null 2>&1; then
@@ -201,7 +236,7 @@ do
 	-t*)	targetgnusystem="$value" ;;
 	-r*)	if [ -n "$value" ]; then remotehost=$value;
 		elif [ $# -lt 1 ]; then
-		    echo "$0: -r option missing argument!" >&2
+		    echo "$PROGNAME: -r option missing argument!" >&2
 		    usage >&2; exit 1;
 		else shift; remotehost=$1;
 		fi
@@ -213,10 +248,14 @@ do
 		    set -- "$@" "$filepart"
 		fi
 		;;
-	-h|--help)	usage; exit 0 ;;
+	--no-conf|--noconf)
+		echo "$PROGNAME: $1 is only acceptable as the first command-line option!" >&2
+		exit 1 ;;
+	-h|--help)
+		usage; exit 0 ;;
 	--version)
 		version; exit 0 ;;
-	-*)	echo "$0: Unrecognised option: $1" >&2
+	-*)	echo "$PROGNAME: Unrecognised option: $1" >&2
 		usage >&2; exit 1 ;;
 	*)	break ;;
     esac
@@ -253,18 +292,18 @@ case $# in
 		dsc=`echo $changes | \
 		    perl -pe 's/\.changes$/.dsc/; s/(.*)_(.*)_(.*)\.dsc/\1_\2.dsc/'`
 		;;
-	    *)	echo "$0: Only a .changes or .dsc file is allowed as argument!" >&2
+	    *)	echo "$PROGNAME: Only a .changes or .dsc file is allowed as argument!" >&2
 		exit 1 ;;
 	esac
 	;;
 
     0)	# We have to parse debian/changelog to find the current version
 	if [ -n "$remotehost" ]; then
-	    echo "$0: Need to specify a .changes or .dsc file location with -r!" >&2
+	    echo "$PROGNAME: Need to specify a .changes or .dsc file location with -r!" >&2
 	    exit 1
 	fi
 	if [ ! -r debian/changelog ]; then
-	    echo "$0: Must be run from top of source dir or a .changes file given as arg" >&2
+	    echo "$PROGNAME: Must be run from top of source dir or a .changes file given as arg" >&2
 	    exit 1
 	fi
 
@@ -275,11 +314,6 @@ case $# in
 
 	if [ "x$sourceonly" = x ]
 	then
-	    if ! command -v dpkg-architecture > /dev/null 2>&1; then
-		echo "This program depends on dpkg-architecture; your dpkg is far too old" >& 1
-		exit 1
-	    fi
-
 	    mustsetvar arch "`dpkg-architecture -a${targetarch} -t${targetgnusystem} -qDEB_HOST_ARCH`" "build architecture"
 	else
 	    arch=source
@@ -292,14 +326,14 @@ case $# in
 	changes="../$pva.changes"
 	;;
 
-    *)	echo "$0: Only a single .changes or .dsc file is allowed as argument!" >&2
+    *)	echo "$PROGNAME: Only a single .changes or .dsc file is allowed as argument!" >&2
 	exit 1 ;;
 esac
 
 # Do we have to download the changes file?
 if [ -n "$remotehost" ]
 then
-    mkdir debsign.$$ || { echo "$0: Can't mkdir!" >&2; exit 1; }
+    mkdir debsign.$$ || { echo "$PROGNAME: Can't mkdir!" >&2; exit 1; }
     trap "cleanup_tmpdir" 0 1 2 3 7 10 13 15
     cd debsign.$$
     remotechanges=$changes
@@ -317,7 +351,7 @@ if [ -n "$changes" ]
 then
     if [ ! -f "$changes" -o ! -r "$changes" ]
     then
-	echo "$0: Can't find or can't read changes file $changes!" >&2
+	echo "$PROGNAME: Can't find or can't read changes file $changes!" >&2
 	exit 1
     fi
 
@@ -346,7 +380,7 @@ then
 
 	if [ ! -f "$dsc" -o ! -r "$dsc" ]
 	then
-	    echo "$0: Can't find or can't read dsc file $dsc!" >&2
+	    echo "$PROGNAME: Can't find or can't read dsc file $dsc!" >&2
 	    exit 1
 	fi
 	check_already_signed "$dsc" "dsc" || withecho signfile "$dsc" "$signas"
@@ -388,7 +422,7 @@ then
 else # only a dsc file to sign; much easier
     if [ ! -f "$dsc" -o ! -r "$dsc" ]
     then
-	echo "$0: Can't find or can't read dsc file $dsc!" >&2
+	echo "$PROGNAME: Can't find or can't read dsc file $dsc!" >&2
 	exit 1
     fi
 
