@@ -23,27 +23,40 @@
 
 use 5.004;	# correct pipe close behavior
 use strict;
+use Cwd;
+use File::Basename;
+use POSIX	qw(:errno_h);
 
 BEGIN {
     eval { require Digest::MD5; };
     if ($@) {
-	die "Problem loading the Digest::MD5.pm module: $@\nHave you installed the libdigest-md5-perl package?\n";
+	my $progname = basename $0;
+	if ($@ =~ /^Can\'t locate Digest\/MD5\.pm/) {
+	    die "$progname: you must have the libdigest-md5-perl package installed\nto use this script\n";
+	}
+	die "$progname: problem loading the Digest::MD5 module:\n  $@\nHave you installed the libdigest-md5-perl package?\n";
     }
 }
-use Cwd;
 
-(my $progname = $0) =~ s%.*/%%;
+my $progname = basename $0;
+my $modified_conf_msg;
 my $Exit = 0;
-my $start_dir = cwd();
+my $start_dir = cwd;
 
-my $usage = <<"EOF";
+sub usage {
+    print <<"EOF";
 Usage: $progname [options] dsc-or-changes-file ...
   Options: --help      Display this message
            --version   Display version and copyright information
            --keyring <keyring>
                        Add <keyring> to the list of keyrings used
-           --noconf    Do not read the devscripts config file
+           --no-conf, --noconf
+                       Do not read the devscripts config file
+
+Default settings modified by devscripts configuration files:
+$modified_conf_msg
 EOF
+}
 
 my $version = <<"EOF";
 This is $progname, from the Debian devscripts package, version ###VERSION###
@@ -116,12 +129,17 @@ sub process_file {
 	$filebase = $file;
     }
 
-    unless (-r $filebase) {
-	xwarn "can't read $file:";
+    if (!open SIGNED, $filebase) {
+	xwarn "can't open $file:";
+	return;
+    }
+    my $out = do { local $/; <SIGNED> };
+    if (!close SIGNED) {
+	xwarn "problem reading $file:";
 	return;
     }
 
-    my $sigcheck = check_signature($filebase, @rings);
+    my $sigcheck = check_signature $filebase, @rings;
     if ($sigcheck) {
 	xwarn "$file failed signature check:\n$sigcheck";
 	return;
@@ -129,14 +147,6 @@ sub process_file {
 	print "      Good signature found\n";
     }
 
-    open SIGNED, $filebase or xwarn("can't open $file:"), return;
-    my $out;
-    {
-	local $/;
-	$out = <SIGNED>;
-    }
-    close SIGNED or xwarn("problem reading $file:"), return;
-
     my @spec = map { split /\n/ } $out =~ /^Files:\s*\n((?:[ \t]+.*\n)+)/mg;
     unless (@spec) {
 	xwarn "no file spec lines in $file\n";
@@ -153,7 +163,7 @@ sub process_file {
 	my ($md5, $size, $filename) = ($1, $2, $3);
 
 	unless (open FILE, $filename) {
-	    if ($! == ENOENT()) {
+	    if ($! == ENOENT) {
 		print STDERR "   skipping  $filename (not present)\n";
 	    }
 	    else {
@@ -188,7 +198,7 @@ sub process_file {
 	close FILE;
 
 	if ($filename =~ /\.dsc$/) {
-	    $sigcheck = check_signature($filename, @rings);
+	    $sigcheck = check_signature $filename, @rings;
 	    if ($sigcheck) {
 		xwarn "$filename failed signature check:\n$sigcheck";
 		next;
@@ -210,12 +220,14 @@ sub main {
     # Handle config file unless --no-conf or --noconf is specified
     # The next stuff is boilerplate
     if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
-	shift;
+	$modified_conf_msg = "  (no configuration files read)";
+	shift @ARGV;
     } else {
 	my @config_files = ('/etc/devscripts.conf', '~/.devscripts');
 	my %config_vars = (
 			   'DSCVERIFY_KEYRINGS' => '',
 			   );
+	my %config_default = %config_vars;
 
 	my $shell_cmd;
 	# Set defaults
@@ -226,9 +238,17 @@ sub main {
 	$shell_cmd .= '[ -f $file ] && . $file; done;' . "\n";
 	# Read back values
 	foreach my $var (keys %config_vars) { $shell_cmd .= "echo \$$var;\n" }
-	my $shell_out = `/bin/sh -c '$shell_cmd'`;
+	my $shell_out = `/bin/bash -c '$shell_cmd'`;
 	@config_vars{keys %config_vars} = split /\n/, $shell_out, -1;
 
+	foreach my $var (sort keys %config_vars) {
+	    if ($config_vars{$var} ne $config_default{$var}) {
+		$modified_conf_msg .= "  $var=$config_vars{$var}\n";
+	    }
+	}
+	$modified_conf_msg ||= "  (none)\n";
+	chomp $modified_conf_msg;
+
 	$config_vars{'DSCVERIFY_KEYRINGS'} =~ s/^\s*:\s*//;
 	$config_vars{'DSCVERIFY_KEYRINGS'} =~ s/\s*:\s*$//;
 	@rings = split /\s*:\s*/, $config_vars{'DSCVERIFY_KEYRINGS'};
@@ -236,8 +256,11 @@ sub main {
 
     ## handle command-line options
     while (@ARGV > 0) {
-	if ($ARGV[0] eq '--help') { print $usage; exit 0; }
+	if ($ARGV[0] eq '--help') { usage; exit 0; }
 	if ($ARGV[0] eq '--version') { print $version; exit 0; }
+	if ($ARGV[0] =~ /^--no-?conf$/) {
+	    xdie "$ARGV[0] is only acceptable as the first command-line option!\n";
+	}
 	if ($ARGV[0] eq '--keyring') {
 	    shift @ARGV;
 	    if (@ARGV > 0) {
@@ -268,11 +291,10 @@ sub main {
 	}
 	last;
     }
-		
 
     @ARGV or xdie "no .changes or .dsc files specified\n";
 
-    @rings = get_rings(@rings);
+    @rings = get_rings @rings;
 
     for my $file (@ARGV) {
 	process_file $file, @rings;
