@@ -23,28 +23,40 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-use 5.6.0;  # uses 'our' variables
+use 5.006_000;  # uses 'our' variables
 use strict;
 use Cwd;
 use File::Basename;
+use Getopt::Long;
 use lib '/usr/share/devscripts';
 use Devscripts::Versort;
 BEGIN {
     eval { require LWP::UserAgent; };
-    if ($@ =~ /^Can\'t locate LWP\/UserAgent\.pm/) {
-	die "You must have the libwww-perl package installed to use this script\n";
+    if ($@) {
+	my $progname = basename($0);
+	if ($@ =~ /^Can\'t locate LWP\/UserAgent\.pm/) {
+	    die "$progname: you must have the libwww-perl package installed\nto use this script\n";
+	} else {
+	    die "$progname: problem loading the LWP::UserAgent module:\n  $@\nHave you installed the libwww-perl package?\n";
+	}
     }
 }
 
-sub process_watchline ($$$$$);
-sub process_watchfile ($$$);
+my $progname = basename($0);
+my $modified_conf_msg;
+my $opwd = cwd();
 
-(my $progname = $0) =~ s|.*/||;
+# Did we find any new upstream versions on our wanderings?
+our $found = 0;
 
-my $usage = <<"EOF";
-Usage: $progname [options] dir ...
-  Process watchfiles in all .../debian/ subdirs of those listed
-  to check for upstream releases.
+sub process_watchline ($$$$$$);
+sub process_watchfile ($$$$);
+
+sub usage {
+    print <<"EOF";
+Usage: $progname [options] [dir ...]
+  Process watchfiles in all .../debian/ subdirs of those listed (or the
+  current directory if none listed) to check for upstream releases.
 Options:
     --report, --no-download
                    Only report on newer or absent versions, do not download
@@ -55,11 +67,29 @@ Options:
     --no-symlink   Don\'t make this symlink
     --verbose      Give verbose output
     --no-verbose   Don\'t give verbose output (default)
+    --check-dirname-level N
+                   How much to check directory names:
+                   N=0   never
+                   N=1   only when program changes directory (default)
+                   N=2   always
+    --check-dirname-regex REGEX
+                   What constitutes a matching directory name; REGEX is
+                   a Perl regular expression; the string \`PACKAGE\' will
+                   be replaced by the package name; see manpage for details
+                   (default: 'PACKAGE(-.*)?')
+    --no-conf, --noconf
+                   Don\'t read devscripts config files;
+                   must be the first option given
     --help         Show this message
     --version      Show version information
-EOF
 
-my $version = <<"EOF";
+Default settings modified by devscripts configuration files:
+$modified_conf_msg
+EOF
+}
+
+sub version {
+    print <<"EOF";
 This is $progname, from the Debian devscripts package, version ###VERSION###
 This code is copyright 1999 by Julian Gilbey, all rights reserved.
 Original code by Christoph Lameter.
@@ -67,62 +97,124 @@ This program comes with ABSOLUTELY NO WARRANTY.
 You are free to redistribute this code under the terms of the
 GNU General Public License, version 2 or later.
 EOF
+}
 
 # What is the default setting of $ENV{'FTP_PASSIVE'}?
-our $passive;
+our $passive = 'default';
 
 # Now start by reading configuration files and then command line
 # The next stuff is boilerplate
 
-my @config_files = ('/etc/devscripts.conf', '~/.devscripts');
-my %config_vars = (
-		   'USCAN_DOWNLOAD' => 'yes',
-		   'USCAN_PASV' => 'default',
-		   'USCAN_SYMLINK' => 'yes',
-		   'USCAN_VERBOSE' => 'no',
-		   );
+my $download = 1;
+my $symlink = 1;
+my $verbose = 0;
+my $check_dirname_level = 1;
+my $check_dirname_regex = 'PACKAGE(-.*)?';
 
-my $shell_cmd;
-# Set defaults
-foreach my $var (keys %config_vars) {
-    $shell_cmd .= "$var='$config_vars{$var}';\n";
+if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
+    $modified_conf_msg = "  (no configuration files read)";
+    shift;
+} else {
+    my @config_files = ('/etc/devscripts.conf', '~/.devscripts');
+    my %config_vars = (
+		       'USCAN_DOWNLOAD' => 'yes',
+		       'USCAN_PASV' => 'default',
+		       'USCAN_SYMLINK' => 'yes',
+		       'USCAN_VERBOSE' => 'no',
+		       'DEVSCRIPTS_CHECK_DIRNAME_LEVEL' => 1,
+		       'DEVSCRIPTS_CHECK_DIRNAME_REGEX' => 'PACKAGE(-.*)?',
+		       );
+    my %config_default = %config_vars;
+
+    my $shell_cmd;
+    # Set defaults
+    foreach my $var (keys %config_vars) {
+	$shell_cmd .= qq[$var="$config_vars{$var}";\n];
+    }
+    $shell_cmd .= 'for file in ' . join(" ",@config_files) . "; do\n";
+    $shell_cmd .= '[ -f $file ] && . $file; done;' . "\n";
+    # Read back values
+    foreach my $var (keys %config_vars) { $shell_cmd .= "echo \$$var;\n" }
+    my $shell_out = `/bin/bash -c '$shell_cmd'`;
+    @config_vars{keys %config_vars} = split /\n/, $shell_out, -1;
+
+    # Check validity
+    $config_vars{'USCAN_DOWNLOAD'} =~ /^(yes|no)$/
+	or $config_vars{'USCAN_DOWNLOAD'}='yes';
+    $config_vars{'USCAN_PASV'} =~ /^(yes|no|default)$/
+	or $config_vars{'USCAN_PASV'}='default';
+    $config_vars{'USCAN_SYMLINK'} =~ /^(yes|no)$/
+	or $config_vars{'USCAN_SYMLINK'}='yes';
+    $config_vars{'USCAN_VERBOSE'} =~ /^(yes|no)$/
+	or $config_vars{'USCAN_VERBOSE'}='no';
+    $config_vars{'DEVSCRIPTS_CHECK_DIRNAME_LEVEL'} =~ /^[012]$/
+	or $config_vars{'DEVSCRIPTS_CHECK_DIRNAME_LEVEL'}=1;
+
+    foreach my $var (sort keys %config_vars) {
+	if ($config_vars{$var} ne $config_default{$var}) {
+	    $modified_conf_msg .= "  $var=$config_vars{$var}\n";
+	}
+    }
+    $modified_conf_msg ||= "  (none)\n";
+    chomp $modified_conf_msg;
+
+    $download = $config_vars{'USCAN_DOWNLOAD'} eq 'no' ? 0 : 1;
+    $passive = $config_vars{'USCAN_PASV'} eq 'yes' ? 1 :
+	$config_vars{'USCAN_PASV'} eq 'no' ? 0 : 'default';
+    $symlink = $config_vars{'USCAN_SYMLINK'} eq 'no' ? 0 : 1;
+    $verbose = $config_vars{'USCAN_VERBOSE'} eq 'yes' ? 1 : 0;
+    $check_dirname_level = $config_vars{'DEVSCRIPTS_CHECK_DIRNAME_LEVEL'};
+    $check_dirname_regex = $config_vars{'DEVSCRIPTS_CHECK_DIRNAME_REGEX'};
 }
-$shell_cmd .= 'for file in ' . join(" ",@config_files) . "; do\n";
-$shell_cmd .= '[ -f $file ] && . $file; done;' . "\n";
-# Read back values
-foreach my $var (keys %config_vars) { $shell_cmd .= "echo \$$var;\n" }
-my $shell_out = `/bin/sh -c '$shell_cmd'`;
-@config_vars{keys %config_vars} = split /\n/, $shell_out, -1;
-
-my $download = $config_vars{'USCAN_DOWNLOAD'} eq 'no' ? 0 : 1;
-$passive = $config_vars{'USCAN_PASV'} eq 'yes' ? 1 :
-    $config_vars{'USCAN_PASV'} eq 'no' ? 0 : 'default';
-my $symlink = $config_vars{'USCAN_SYMLINK'} eq 'no' ? 0 : 1;
-my $verbose = $config_vars{'USCAN_VERBOSE'} eq 'yes' ? 1 : 0;
-my $debug = 0;
 
 # Now read the command line arguments
-while (@ARGV) {
-    my $ARG = shift @ARGV;
-    if ($ARG eq '--help')          { print $usage; exit 0; }
-    if ($ARG eq '--version')       { print $version; exit 0; }
-    if ($ARG eq '--download')      { $download=1; next; }
-    if ($ARG =~ /^--no-?download$/) { $download=0; next; }
-    if ($ARG eq '--report')        { $download=0; next; }
-    if ($ARG eq '--pasv')          { $passive=1; next; }
-    if ($ARG =~ /^--no-?pasv$/)    { $passive=0; next; }
-    # Be liberal in what you accept
-    if ($ARG eq '--passive')       { $passive=1; next; }
-    if ($ARG =~ /^--no-?passive$/) { $passive=0; next; }
-    if ($ARG eq '--symlink')       { $symlink=1; next; }
-    if ($ARG =~ /^--no-?symlink$/) { $symlink=0; next; }
-    if ($ARG eq '--verbose')       { $verbose=1; next; }
-    if ($ARG =~ /^--no-?verbose$/) { $verbose=1; next; }
-    if ($ARG eq '--debug')         { $debug=1; next; }
-    # Not an option, so
-    unshift @ARGV, $ARG;
-    last;
+my $debug = 0;
+my ($opt_h, $opt_v, $opt_download, $opt_passive, $opt_symlink);
+my ($opt_verbose, $opt_ignore, $opt_level, $opt_regex, $opt_noconf);
+
+GetOptions("help" => \$opt_h,
+	   "version" => \$opt_v,
+	   "download!" => \$opt_download,
+	   "report" => sub { $opt_download = 0; },
+	   "passive|pasv!" => \$opt_passive,
+	   "symlink!" => \$opt_symlink,
+	   "verbose!" => \$opt_verbose,
+	   "debug" => \$debug,
+	   "ignore-dirname" => \$opt_ignore,
+	   "check-dirname-level=s" => \$opt_level,
+	   "check-dirname-regex=s" => \$opt_regex,
+	   "noconf" => \$opt_noconf,
+	   "no-conf" => \$opt_noconf,
+	   )
+    or die "Usage: $progname [options] [directories]\nRun $progname --help for more details\n";
+
+if ($opt_noconf) {
+    die "$progname: --no-conf is only acceptable as the first command-line option!\n";
 }
+if ($opt_h) { usage(); exit 0; }
+if ($opt_v) { version(); exit 0; }
+
+# Now we can set the other variables according to the command line options
+
+$download = $opt_download if defined $opt_download;
+$passive = $opt_passive if defined $opt_passive;
+$symlink = $opt_symlink if defined $opt_symlink;
+$verbose = $opt_verbose if defined $opt_verbose;
+
+# dirname stuff
+if ($opt_ignore) {
+    die "$progname: --ignore-dirname has been replaced by --check-dirname-level and\n--check-dirname-regex; run $progname --help for more details\n";
+}
+
+if (defined $opt_level) {
+    if ($opt_level =~ /^[012]$/) { $check_dirname_level = $opt_level; }
+    else {
+	die "$progname: unrecognised --check-dirname-level value (allowed are 0,1,2)\n";
+    }
+}
+
+$check_dirname_regex = $opt_regex if defined $opt_regex;
+
 
 # We'd better be verbose if we're debugging
 $verbose |= $debug;
@@ -152,7 +244,7 @@ print "-- Scanning for watchfiles in @ARGV\n" if $verbose;
 my @dirs;
 my $pid = open FIND, '-|';
 if (! defined $pid) {
-    die "Couldn't fork: $!\n";
+    die "$progname: couldn't fork: $!\n";
 }
 if ($pid) {
     while (<FIND>) {
@@ -162,18 +254,20 @@ if ($pid) {
     close FIND;
 } else {
     exec 'find', @ARGV, qw(-type d -name debian -print);
-    die "Couldn't exec find: $!\n";
+    die "$progname: couldn't exec find: $!\n";
 }
 
+my @debdirs = ();
+
 my $origdir = cwd;
 for my $dir (@dirs) {
     unless (chdir $origdir) {
-	warn "Warning: Couldn't chdir back to $origdir, skipping: $!\n";
+	warn "$progname warning: Couldn't chdir back to $origdir, skipping: $!\n";
 	next;
     }
     $dir =~ s%/debian$%%;
     unless (chdir $dir) {
-	warn "Warning: Couldn't chdir $dir, skipping: $!\n";
+	warn "$progname warning: Couldn't chdir $dir, skipping: $!\n";
 	next;
     }
 
@@ -183,39 +277,86 @@ for my $dir (@dirs) {
 	# Figure out package info we need
 	my $changelog = `dpkg-parsechangelog`;
 	unless ($? == 0) {
-	    warn "Warning: Problems running dpkg-parsechangelog in $dir, skipping\n";
+	    warn "$progname warning: Problems running dpkg-parsechangelog in $dir, skipping\n";
 	    next;
 	}
 
-	my ($package, $version);
+	my ($package, $debversion, $version);
 	$changelog =~ /^Source: (.*?)$/m and $package=$1;
-	$changelog =~ /^Version: (.*?)$/m and $version=$1;
-	# Get upstream version number
-	if ($version) {
-	    $version =~ s/-[^-]+$//;
-	    $version =~ s/^\d+://;
-	}
-
-	if ($package and $version) {
-	    process_watchfile($dir, $package, $version);
-	} else {
-	    warn "Warning: Problems determining package name and version from\n  $dir/debian/changelog, skipping\n";
+	$changelog =~ /^Version: (.*?)$/m and $debversion=$1;
+	if (! defined $package || ! defined $debversion) {
+	    warn "$progname warning: Problems determining package name and/or version from\n  $dir/debian/changelog, skipping\n";
 	    next;
 	}
+	
+	# Check the directory is properly named for safety
+	my $good_dirname;
+	if ($check_dirname_level ==  2 or
+	    ($check_dirname_level == 1 and cwd() ne $opwd)) {
+	    my $re = $check_dirname_regex;
+	    $re =~ s/PACKAGE/\\Q$package\\E/g;
+	    if ($re =~ m%/%) { $good_dirname = eval "cwd() =~ /^$re\$/;"; }
+	    else { $good_dirname = eval "basename(cwd()) =~ /^$re\$/;"; }
+	} else {
+	    $good_dirname = 1;
+	}
+
+	# Get upstream version number
+	$version = $debversion;
+	$version =~ s/-[^-]+$//;  # revision
+	$version =~ s/^\d+://;    # epoch
+
+	push @debdirs, [$debversion, $dir, $package, $version, $good_dirname];
     }
     elsif (-r 'debian/watch') {
-	warn "Warning: Found watchfile in $dir,\n  but couldn't find/read changelog; skipping\n";
+	warn "$progname warning: Found watchfile in $dir,\n  but couldn't find/read changelog; skipping\n";
 	next;
     }
     elsif (-f 'debian/watch') {
-	warn "Warning: Found watchfile in $dir,\n  but it is not readable; skipping\n";
+	warn "$progname warning: Found watchfile in $dir,\n  but it is not readable; skipping\n";
 	next;
     }
 }
 
+# Now sort the list of directories, so that we process the most recent
+# directories first, as determined by the package version numbers
+@debdirs = Devscripts::Versort::deb_versort(@debdirs);
+
+# Now process the watchfiles in order.  If a directory d has subdirectories
+# d/sd1/debian and d/sd2/debian, which each contain watchfiles corresponding
+# to the same package, then we only process the watchfile in the package with
+# the latest version number.
+my %donepkgs;
+for my $debdir (@debdirs) {
+    shift @$debdir;  # don't need the Debian version number any longer
+    my $dir = $$debdir[0];
+    my $parentdir = dirname($dir);
+    my $package = $$debdir[1];
+    my $version = $$debdir[2];
+    my $good_dirname = $$debdir[3];
+
+    if (exists $donepkgs{$parentdir}{$package}) {
+	warn "$progname warning: Skipping $dir/debian/watch\n  as this package has already been scanned successfully\n";
+	next;
+    }
+
+    unless (chdir $origdir) {
+	warn "$progname warning: Couldn't chdir back to $origdir, skipping: $!\n";
+	next;
+    }
+    unless (chdir $dir) {
+	warn "$progname warning: Couldn't chdir $dir, skipping: $!\n";
+	next;
+    }
+
+    if (process_watchfile($dir, $package, $version, $good_dirname) == 0) {
+	$donepkgs{$parentdir}{$package} = 1;
+    }
+}
+
 print "-- Scan finished\n" if $verbose;
 
-exit 0;
+exit $found ? 0 : 1;
 
 
 # This is the heart of the code: Process a single watch item
@@ -244,9 +385,9 @@ exit 0;
 # greatest version number (as determined by the (...) group), using the
 # Debian version number comparison algorithm described below.
 
-sub process_watchline ($$$$$)
+sub process_watchline ($$$$$$)
 {
-    my ($line, $watch_version, $pkg_dir, $pkg, $pkg_version) = @_;
+    my ($line, $watch_version, $pkg_dir, $pkg, $pkg_version, $good_dirname) = @_;
 
     my ($base, $site, $dir, $pattern, $lastversion, $action);
     my %options = ();
@@ -260,8 +401,8 @@ sub process_watchline ($$$$$)
 	($site, $dir, $pattern, $lastversion, $action) = split ' ', $line;
 
 	if (! defined $lastversion or $site =~ /\(.*\)/ or $dir =~ /\(.*\)/) {
-	    warn "Warning: there appears to be a version 2 format line in\n  the version 1 watchfile $dir/debian/watch;\n  Have you forgotten a 'version=2' line at the start, perhaps?\n  Skipping the line: $line\n";
-	    return;
+	    warn "$progname warning: there appears to be a version 2 format line in\n  the version 1 watchfile $dir/debian/watch;\n  Have you forgotten a 'version=2' line at the start, perhaps?\n  Skipping the line: $line\n";
+	    return 1;
 	}
 	if ($site !~ m%\w+://%) {
 	    $site = "ftp://$site";
@@ -275,7 +416,7 @@ sub process_watchline ($$$$$)
 		$pattern =~ s/\?/./g;
 		$pattern =~ s/\*/.*/g;
 		$style='old';
-		warn "Warning: Using very old style of filename pattern in $pkg_dir/debian/watch\n  (this might lead to incorrect results): $3\n";
+		warn "$progname warning: Using very old style of filename pattern in $pkg_dir/debian/watch\n  (this might lead to incorrect results): $3\n";
 	    }
 	}
 
@@ -297,7 +438,7 @@ sub process_watchline ($$$$$)
 		    $options{'pasv'}=0;
 		}
 		else {
-		    warn "Warning: unrecognised option $opt\n";
+		    warn "$progname warning: unrecognised option $opt\n";
 		}
 	    }
 	}
@@ -318,31 +459,31 @@ sub process_watchline ($$$$$)
 	if ($base =~ m%^(\w+://[^/]+)%) {
 	    $site = $1;
 	} else {
-	    warn "Warning: Can't determine protocol and site in\n  $pkg_dir/debian/watch, skipping:\n  $line\n";
-	    return;
+	    warn "$progname warning: Can't determine protocol and site in\n  $pkg_dir/debian/watch, skipping:\n  $line\n";
+	    return 1;
 	}
     }
 
     # Check all's OK
     if ($pattern !~ /\(.*\)/) {
-	warn "Warning: Filename pattern missing version delimeters ()\n  in $pkg_dir/debian/watch, skipping:\n  $line\n";
-	return;
+	warn "$progname warning: Filename pattern missing version delimeters ()\n  in $pkg_dir/debian/watch, skipping:\n  $line\n";
+	return 1;
     }
 
     # What is the most recent file, based on the filenames?
     # We first have to find the candidates, then we sort them using
     # Devscripts::Versort::versort
     if ($site =~ m%^http://%) {
-	print STDERR "Debug: requesting URL $base\n" if $debug;
+	print STDERR "$progname debug: requesting URL $base\n" if $debug;
 	$request = HTTP::Request->new('GET', $base);
 	$response = $user_agent->request($request);
 	if (! $response->is_success) {
-	    warn "Warning: In watchfile $pkg_dir/debian/watch, reading webpage\n  $base failed: " . $response->status_line . "\n";
-	    return;
+	    warn "$progname warning: In watchfile $pkg_dir/debian/watch, reading webpage\n  $base failed: " . $response->status_line . "\n";
+	    return 1;
 	}
 
 	my $content = $response->content;
-	print STDERR "Debug: received content:\n$content\[End of received content]\n"
+	print STDERR "$progname debug: received content:\n$content\[End of received content]\n"
 	    if $debug;
 	# We need this horrid stuff to handle href=foo type
 	# links.  OK, bad HTML, but we have to handle it nonetheless.
@@ -360,7 +501,7 @@ sub process_watchline ($$$$$)
 	    ($urlbase = $base) =~ s%/[^/]*$%/%;
 	}
 
-	print STDERR "Debug: matching pattern $pattern\n" if $debug;
+	print STDERR "$progname debug: matching pattern $pattern\n" if $debug;
 	my @hrefs;
 	while ($content =~ m/<\s*a\s+[^>]*href\s*=\s*([\"\'])(.*?)\1/gi) {
 	    my $href = $2;
@@ -376,21 +517,21 @@ sub process_watchline ($$$$$)
 	    @hrefs = Devscripts::Versort::versort(@hrefs);
 	    ($newversion, $newfile) = @{$hrefs[0]};
 	} else {
-	    warn "Warning: In $pkg_dir/debian/watch,\n  no matching hrefs for watch line\n  $line\n";
-	    return;
+	    warn "$progname warning: In $pkg_dir/debian/watch,\n  no matching hrefs for watch line\n  $line\n";
+	    return 1;
 	}
     }
     else {
 	# Better be an FTP site
 	if ($site !~ m%^ftp://%) {
-	    warn "Warning: Unknown protocol in $pkg_dir/debian/watch, skipping:\n  $site\n";
-	    return;
+	    warn "$progname warning: Unknown protocol in $pkg_dir/debian/watch, skipping:\n  $site\n";
+	    return 1;
 	}
 
 	if (exists $options{'pasv'}) {
 	    $ENV{'FTP_PASSIVE'}=$options{'pasv'};
 	}
-	print STDERR "Debug: requesting URL $base\n" if $debug;
+	print STDERR "$progname debug: requesting URL $base\n" if $debug;
 	$request = HTTP::Request->new('GET', $base);
 	$response = $user_agent->request($request);
 	if (exists $options{'pasv'}) {
@@ -398,19 +539,19 @@ sub process_watchline ($$$$$)
 	    else { delete $ENV{'FTP_PASSIVE'}; }
 	}
 	if (! $response->is_success) {
-	    warn "Warning: In watchfile $pkg_dir/debian/watch, reading FTP directory\n  $base failed: " . $response->status_line . "\n";
-	    return;
+	    warn "$progname warning: In watchfile $pkg_dir/debian/watch, reading FTP directory\n  $base failed: " . $response->status_line . "\n";
+	    return 1;
 	}
 
 	my $content = $response->content;
-	print STDERR "Debug: received content:\n$content\[End of received content]\n"
+	print STDERR "$progname debug: received content:\n$content\[End of received content]\n"
 	    if $debug;
 
 	# FTP directory listings either look like:
 	# info info ... info filename [ -> linkname]
 	# or they're HTMLised (if they've been through an HTTP proxy)
 	# so we may have to look for <a href="filename"> type patterns
-	print STDERR "Debug: matching pattern $pattern\n" if $debug;
+	print STDERR "$progname debug: matching pattern $pattern\n" if $debug;
 	my (@files);
 	$content =~ s/\n/ \n/g; # make every filename have an extra
 	                        # space after it in a normal FTP listing
@@ -426,8 +567,8 @@ sub process_watchline ($$$$$)
 	    @files = Devscripts::Versort::versort(@files);
 	    ($newversion, $newfile) = @{$files[0]};
 	} else {
-	    warn "Warning: In $pkg_dir/debian/watch no matching files for watch line\n  $line\n";
-	    return;
+	    warn "$progname warning: In $pkg_dir/debian/watch no matching files for watch line\n  $line\n";
+	    return 1;
 	}
     }
 
@@ -442,13 +583,13 @@ sub process_watchline ($$$$$)
 	    $newversion = $1;
 	} else {
 	    warn <<"EOF";
-Warning: In $pkg_dir/debian/watch, couldn\'t determine a
+$progname warning: In $pkg_dir/debian/watch, couldn\'t determine a
   pure numeric version number from the file name for watch line
   $line
   and file name $newfile
   Please use a new style watchfile instead!
 EOF
-	    return;
+	    return 1;
 	}
     }
 			
@@ -459,9 +600,12 @@ EOF
 
     print "Newest version on remote site is $newversion, local version is $lastversion\n"
 	if $verbose;
-    if ($newversion eq $lastversion) {
+
+    # Can't just use $lastversion eq $newversion, as then 0.01 and 0.1
+    # compare different, whereas they are treated as equal by dpkg
+    if (system("dpkg --compare-versions '$lastversion' eq '$newversion'") == 0) {
 	print " => Package is up to date\n" if $verbose;
-	return;
+	return 0;
     }
 
     # We use dpkg's rules to determine whether our current version
@@ -472,17 +616,21 @@ EOF
 	} else {
 	    print "$pkg: remote site does not even have current version\n";
 	}
-        return;
+        return 0;
+    } else {
+	# There's a newer upstream version available, which may already
+	# be on our system or may not be
+	$found++;
     }
 
     if (-f "../$newfile_base") {
         print " => $newfile_base already in package directory\n"
 	    if $verbose;
-        return;
+        return 0;
     }
     if (-f "../${pkg}_${newversion}.orig.tar.gz") {
-        warn "Warning: In directory $pkg_dir, found file\n  ${pkg}_${newversion}.orig.tar.gz but not $newfile_base,\n  which is the newest file available on remote site.  Skipping.\n";
-        return;
+        warn "$progname warning: In directory $pkg_dir, found file\n  ${pkg}_${newversion}.orig.tar.gz but not $newfile_base,\n  which is the newest file available on remote site.  Skipping.\n";
+        return 0;
     }
 
     if ($verbose) {
@@ -491,39 +639,39 @@ EOF
 	print "$pkg: Newer version ($newversion) available on remote site\n  (local version is $lastversion)\n";
     }
 
-    return unless $download;
+    return 0 unless $download;
 
     print "-- Downloading updated package $newfile_base\n" if $verbose;
     # Download newer package
     if ($site =~ m%^http://%) {
 	# absolute URL?
 	if ($newfile =~ m%^\w+://%) {
-	    print STDERR "Debug: requesting URL $newfile\n" if $debug;
+	    print STDERR "$progname debug: requesting URL $newfile\n" if $debug;
 	    $request = HTTP::Request->new('GET', $newfile);
 	    $response = $user_agent->request($request, "../$newfile_base");
 	    if (! $response->is_success) {
-		warn "Warning: In directory $pkg_dir, downloading\n  $newfile failed: " . $response->status_line . "\n";
-		return;
+		warn "$progname warning: In directory $pkg_dir, downloading\n  $newfile failed: " . $response->status_line . "\n";
+		return 1;
 	    }
 	}
 	# absolute filename?
 	elsif ($newfile =~ m%^/%) {
-	    print STDERR "Debug: requesting URL $site$newfile\n" if $debug;
+	    print STDERR "$progname debug: requesting URL $site$newfile\n" if $debug;
 	    $request = HTTP::Request->new('GET', "$site$newfile");
 	    $response = $user_agent->request($request, "../$newfile_base");
 	    if (! $response->is_success) {
-		warn "Warning: In directory $pkg_dir, downloading\n  $site$newfile failed: " . $response->status_line . "\n";
-		return;
+		warn "$progname warning: In directory $pkg_dir, downloading\n  $site$newfile failed: " . $response->status_line . "\n";
+		return 1;
 	    }
 	}
 	# relative filename, we hope
 	else {
-	    print STDERR "Debug: requesting URL $urlbase$newfile\n" if $debug;
+	    print STDERR "$progname debug: requesting URL $urlbase$newfile\n" if $debug;
 	    $request = HTTP::Request->new('GET', "$urlbase$newfile");
 	    $response = $user_agent->request($request, "../$newfile_base");
 	    if (! $response->is_success) {
-		warn "Warning: In directory $pkg_dir, downloading\n  $urlbase$newfile failed: " . $response->status_line . "\n";
-		return;
+		warn "$progname warning: In directory $pkg_dir, downloading\n  $urlbase$newfile failed: " . $response->status_line . "\n";
+		return 1;
 	    }
 	}
     }
@@ -532,7 +680,7 @@ EOF
 	if (exists $options{'pasv'}) {
 	    $ENV{'FTP_PASSIVE'}=$options{'pasv'};
 	}
-	print STDERR "Debug: requesting URL $base$newfile\n" if $debug;
+	print STDERR "$progname debug: requesting URL $base$newfile\n" if $debug;
 	$request = HTTP::Request->new('GET', "$base$newfile");
 	$response = $user_agent->request($request, "../$newfile_base");
 	if (exists $options{'pasv'}) {
@@ -540,8 +688,8 @@ EOF
 	    else { delete $ENV{'FTP_PASSIVE'}; }
 	}
 	if (! $response->is_success) {
-	    warn "Warning: In directory $pkg_dir, downloading\n  $base$newfile failed: " . $response->status_line . "\n";
-	    return;
+	    warn "$progname warning: In directory $pkg_dir, downloading\n  $base$newfile failed: " . $response->status_line . "\n";
+	    return 1;
 	}
     }
 
@@ -563,29 +711,47 @@ EOF
 
     # Do whatever the user wishes to do
     if ($action) {
-	my $usefile = ($symlink and $newfile_base =~ /\.(tar\.gz|tgz)$/) ?
-	    "../${pkg}_${newversion}.orig.tar.gz" : "../$newfile_base";
+	if ($good_dirname) {
+	    my $usefile = ($symlink and $newfile_base =~ /\.(tar\.gz|tgz)$/) ?
+		"../${pkg}_${newversion}.orig.tar.gz" : "../$newfile_base";
 
-	if ($watch_version > 1) {
-	    print "-- Executing user specified script\n     $action --upstream-version $newversion $newfile_base" if $verbose;
-	    system("$action --upstream-version $newversion $usefile");
-	} else {
-	    print "-- Executing user specified script $action $newfile_base $newversion" if $verbose;
-	    system("$action $usefile $newversion");
+	    if ($watch_version > 1) {
+		print "-- Executing user specified script\n     $action --upstream-version $newversion $newfile_base" if $verbose;
+		system("$action --upstream-version $newversion $usefile");
+	    } else {
+		print "-- Executing user specified script $action $newfile_base $newversion" if $verbose;
+		system("$action $usefile $newversion");
+	    }
+	}
+	else {
+	    warn <<"EOF";
+$progname warning: An action was requested by the watchfile debian/watch
+for the package $pkg in the directory $pkg_dir
+but this directory name does not match the package name.
+So no action is being performed for safety reasons.
+
+To allow actions to be run in future, see the --check-dirname-level and
+--check-dirname-regex options; run $progname --help for more info.
+
+EOF
 	}
     }
+
+    return 0;
 }
 
-# parameters are dir, package, upstream version
-sub process_watchfile ($$$)
+# parameters are dir, package, upstream version, good dirname
+sub process_watchfile ($$$$)
 {
-    my ($dir, $package, $version) = @_;
+    my ($dir, $package, $version, $good_dirname) = @_;
     my $watch_version=0;
+    my $status=0;
 
     unless (open WATCH, 'debian/watch') {
-	warn "Warning: could not open $dir/debian/watch: $!\n";
-	return;
+	warn "$progname warning: could not open $dir/debian/watch: $!\n";
+	return 1;
     }
+
     while (<WATCH>) {
 	next if /^\s*\#/;
 	next if /^\s*$/;
@@ -595,7 +761,8 @@ sub process_watchfile ($$$)
 	chomp;
 	if (s/(?<!\\)\\$//) {
 	    if (eof(WATCH)) {
-		warn "Warning: $dir/debian/watch ended with \\; skipping last line\n";
+		warn "$progname warning: $dir/debian/watch ended with \\; skipping last line\n";
+		$status=1;
 		last;
 	    }
 	    $_ .= <WATCH>;
@@ -605,9 +772,13 @@ sub process_watchfile ($$$)
 	if (! $watch_version) {
 	    if (/^version\s*=\s*(\d+)(\s|$)/) {
 		$watch_version=$1;
+		if ($watch_version < 2 or $watch_version > 2) {
+		    print STDERR "Error: $dir/debian/watch version number is unrecognised; skipping watchfile\n";
+		    last;
+		}
 		next;
 	    } else {
-		print "Note: $dir/debian/watch is a version 1 watchfile;\n  consider upgrading to version 2 (see uscan(1) for details).\n" if $verbose;
+		warn "$progname warning: $dir/debian/watch is an obsolete version 1 watchfile;\n  please upgrade to a higher version\n  (see uscan(1) for details).\n";
 		$watch_version=1;
 	    }
 	}
@@ -615,8 +786,12 @@ sub process_watchfile ($$$)
 	# Handle shell \\ -> \
 	s/\\\\/\\/g if $watch_version==1;
 	print "-- In $dir/debian/watch, processing watchfile line:\n   $_\n" if $verbose;
-	process_watchline($_, $watch_version, $dir, $package, $version);
+	$status +=
+	    process_watchline($_, $watch_version, $dir, $package, $version, $good_dirname);
     }
 
-    close WATCH or warn "Warning: problems reading $dir/debian/watch\n";
+    close WATCH or
+	$status=1, warn "$progname warning: problems reading $dir/debian/watch\n";
+
+    return $status;
 }
