@@ -4,7 +4,7 @@
 # Christoph Lameter, December 24, 1996
 # Many modifications by Julian Gilbey <jdg@debian.org> January 1999 onwards
 
-# Copyright 1999, Julian Gilbey
+# Copyright 1999-2003, Julian Gilbey
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -41,14 +41,15 @@
 # Has to be called from within the source archive
 
 PROGNAME=`basename $0`
+MODIFIED_CONF_MSG='Default settings modified by devscripts configuration files:'
 TEMP_DIR="uupdate-temp.$$"
 
 usage () {
     echo \
 "Usage for a new archive:
-  uupdate [options] <new upstream archive> [<version>]
+  $PROGNAME [options] <new upstream archive> [<version>]
 For a patch file:
-  uupdate [options] --patch|-p <patch>[.gz|.bz2]
+  $PROGNAME [options] --patch|-p <patch>[.gz|.bz2]
 Options are:
    --upstream-version <version>, -v <version>
                       specify version number of upstream package
@@ -58,14 +59,23 @@ Options are:
    --pristine, -u     Source is pristine upstream source and should be
                       copied to <pkg>_<version>.orig.tar.gz; not valid
                       for patches
+   --no-symlink       Copy new upstream .tar.gz archive to new location
+                      as <pkg>_<version>.orig.tar.gz instead of making a
+                      symlink
+   --no-conf, --noconf
+                      Don't read devscripts config files;
+                      must be the first option given
+
 $PROGNAME [--help|--version]
-  show this message or give version information."
+  show this message or give version information.
+
+$MODIFIED_CONF_MSG"
 }
 
 version () {
     echo \
 "This is $PROGNAME, from the Debian devscripts package, version ###VERSION###
-This code is copyright 1999 by Julian Gilbey, all rights reserved.
+Copyright 1999-2003, Julian Gilbey <jdg@debian.org>, all rights reserved.
 Original code by Christoph Lameter.
 This program comes with ABSOLUTELY NO WARRANTY.
 You are free to redistribute this code under the terms of the
@@ -75,10 +85,10 @@ GNU General Public License, version 2 or later."
 mustsetvar () {
     if [ "x$2" = x ]
     then
-	echo >&2 "$0: unable to determine $3"
+	echo >&2 "$PROGNAME: unable to determine $3"
 	exit 1
     else
-	# echo "$0: $3 is $2"
+	# echo "$PROGNAME: $3 is $2"
 	eval "$1=\"\$2\""
     fi
 }
@@ -93,59 +103,128 @@ MPATTERN='^(?:[a-zA-Z][a-zA-Z0-9]*(?:-|_|\.))+(\d+\.(?:\d+\.)*\d+)$'
 STATUS=0
 
 # Boilerplate: set config variables
-UUPDATE_ROOTCMD=''
-UUPDATE_PRISTINE=yes
+DEFAULT_UUPDATE_ROOTCMD=
+DEFAULT_UUPDATE_PRISTINE=yes
+DEFAULT_UUPDATE_SYMLINK_ORIG=yes
+VARS="UUPDATE_ROOTCMD UUPDATE_PRISTINE UUPDATE_SYMLINK_ORIG"
 
-for file in /etc/devscripts.conf ~/.devscripts
-do
-    [ -r $file ] && . $file
-done
+if [ "$1" = "--no-conf" -o "$1" = "--noconf" ]; then
+    shift
+    MODIFIED_CONF_MSG="$MODIFIED_CONF_MSG
+  (no configuration files read)"
 
-case "$UUPDATE_PRISTINE" in
-    yes|no) ;;
-    *) UUPDATE_PRISTINE=yes ;;
-esac
+    # set defaults
+    for var in $VARS; do
+	eval "$var=\$DEFAULT_$var"
+    done
+else
+    # Run in a subshell for protection against accidental errors
+    # in the config files
+    eval $(
+	set +e
+	for var in $VARS; do
+	    eval "$var=\$DEFAULT_$var"
+	done
+
+	for file in /etc/devscripts.conf ~/.devscripts
+	  do
+	  [ -r $file ] && . $file
+	done
+
+	set | egrep '^(UUPDATE|DEVSCRIPTS)_')
+
+    # check sanity
+    case "$UUPDATE_PRISTINE" in
+	yes|no) ;;
+	*) UUPDATE_PRISTINE=yes ;;
+    esac
+
+    case "$UUPDATE_SYMLINK_ORIG" in
+	yes|no) ;;
+	*) UUPDATE_SYMLINK_ORIG=yes ;;
+    esac
+
+    # set config message
+    MODIFIED_CONF=''
+    for var in $VARS; do
+	eval "if [ \"\$$var\" != \"\$DEFAULT_$var\" ]; then
+	    MODIFIED_CONF_MSG=\"\$MODIFIED_CONF_MSG
+  $var=\$$var\";
+	MODIFIED_CONF=yes;
+	fi"
+    done
+
+    if [ -z "$MODIFIED_CONF" ]; then
+	MODIFIED_CONF_MSG="$MODIFIED_CONF_MSG
+  (none)"
+    fi
+fi
+
+
+TEMP=$(getopt -s bash -o v:p:r:ubs \
+        --long upstream-version:,patch:,rootcmd: \
+	--long pristine,no-pristine,nopristine \
+	--long symlink,no-symlink,nosymlink \
+	--long no-conf,noconf \
+	--long help,version -n "$PROGNAME" -- "$@")
+
+if [ $? != 0 ] ; then echo "Terminating..." >&2 ; exit 1 ; fi
+
+# Note the quotes around `$TEMP': they are essential!
+eval set -- "$TEMP"
 
 # Process Parameters
 while [ "$1" ]; do
     case $1 in
     --upstream-version|-v)
 	shift; NEW_VERSION="$1" ;;
-    --upstream-version=*)
-	NEW_VERSION=`expr "$1" : '--upstream-version=\(.*\)'` ;;
     --patch|-p)
 	shift; PATCH="$1" ;;
     --rootcmd|-r)
 	shift; UUPDATE_ROOTCMD="$1" ;;
-    --rootcmd=*)
-	UUPDATE_ROOTCMD=`expr "$1" : '--rootcmd=\(.*\)'` ;;
     --pristine|-u)
 	UUPDATE_PRISTINE=yes ;;
     --no-pristine|--nopristine)
 	UUPDATE_PRISTINE=no ;;
+    --symlink|-s)
+	UUPDATE_SYMLINK_ORIG=yes ;;
+    --no-symlink|--nosymlink)
+	UUPDATE_SYMLINK_ORIG=no ;;
+    --no-conf|--noconf)
+	echo "$PROGNAME: $1 is only acceptable as the first command-line option!" >&2
+	exit 1 ;;
     --help) usage; exit 0 ;;
     --version) version; exit 0 ;;
-    -*) echo "Unrecognised option: $1; try uupdate --help for more info" >&2
-	exit 1 ;;
-    *)	ARCHIVE=$1
-	[ "x$2" != x ] && NEW_VERSION="$2"
-	break   # So that we only look at the first non-option argument
-	;;
+    --)	shift; break ;;
+    *) echo "$PROGNAME: bug in option parser, sorry!" >&2 ; exit 1 ;;
     esac
     shift
 done
 
+if [ -n "$PATCH" ]; then
+    if [ $# ne 0 ]; then
+        echo "$PROGNAME: additional archive name/version number is not allowed with --patch" >&2
+	echo "Run $PROGNAME --help for usage information" >&2
+        exit 1
+    fi
+else
+    case $# in
+    0) echo "$PROGNAME: no archive given" >&2 ; exit 1 ;;
+    1) ARCHIVE="$1" ;;
+    2) ARCHIVE="$1"; NEW_VERSION="$2" ;;
+    *) echo "$PROGNAME: too many non-option arguments" >&2
+       echo "Run $PROGNAME --help for usage information" >&2
+       exit 1 ;;
+    esac
+fi
+
 # Get Parameters from current source archive
 
-# Look for debian/changelog; save pwd before we goes walkabout
-OPWD=`pwd`
-until [ -f debian/changelog ]; do
-    cd ..
-    if [ `pwd` = "/" ]; then
-	echo "Cannot find debian/changelog anywhere." >&2
-	exit 1
-    fi
-done
+if [ ! -f debian/changelog ]; then
+    echo "$PROGNAME: cannot find debian/changelog." >&2
+    echo "Are you in the top directory of the source tree?" >&2
+    exit 1
+fi
 
 # Figure out package info we need
 mustsetvar PACKAGE "`dpkg-parsechangelog | sed -n 's/^Source: //p'`" \
@@ -158,13 +237,16 @@ eval `echo "$VERSION" | perl -ne '/^(?:(\d+):)?(.*)/; print "SVERSION=$2\nEPOCH=
 
 UVERSION=`expr "$SVERSION" : '\(.*\)-[0-9a-z.+]*$'` ||
 {
-    echo "A native Debian package cannot take upstream updates" >&2
+    echo "$PROGNAME: a native Debian package cannot take upstream updates" >&2
     exit 1
 }
 
+# Save pwd before we goes walkabout
+OPWD=`pwd`
+
 if [ "$PATCH" ]; then
     if [ "$ARCHIVE" ]; then
-	echo "You can only specify a patch or a source code archive, not both!" >&2
+	echo "$PROGNAME: you can only specify a patch or a source code archive, not both!" >&2
 	exit 1
     fi
     # do the patching
@@ -172,7 +254,7 @@ if [ "$PATCH" ]; then
     case "$PATCH" in
 	/*)
 	    if [ ! -r "$PATCH" ]; then
-		echo "Cannot read patch file $PATCH!  Aborting." >&2
+		echo "$PROGNAME: cannot read patch file $PATCH!  Aborting." >&2
 		exit 1
 	    fi
 	    case "$PATCH" in
@@ -183,7 +265,7 @@ if [ "$PATCH" ]; then
 	    ;;
 	*)
 	    if [ ! -r "$OPWD/$PATCH" -a ! -r "../$PATCH" ]; then
-		echo "Cannot read patch file $PATCH!  Aborting." >&2
+		echo "$PROGNAME: cannot read patch file $PATCH!  Aborting." >&2
 		exit 1
 	    fi
 	    case "$PATCH" in
@@ -217,8 +299,8 @@ if [ "$PATCH" ]; then
 	NEW_VERSION=`echo "$X" |
 		perl -ne 's/\.diff$//; /'"$MPATTERN"'/ && print $1'`
 	if [ -z "$NEW_VERSION" ]; then
-	    echo "New version number not recognized from given filename" >&2
-	    echo "Please run uupdate with the -v option" >&2
+	    echo "$PROGNAME: new version number not recognized from given filename" >&2
+	    echo "Please run $PROGNAME with the -v option" >&2
 	    exit 1
 	fi
 
@@ -237,22 +319,23 @@ if [ "$PATCH" ]; then
 
     # Sanity check
     if dpkg --compare-versions "$NEW_VERSION-1" le "$VERSION"; then
-	echo "New version $NEW_VERSION-1 <= current version $VERSION; aborting!" >&2
+	echo "$PROGNAME: new version $NEW_VERSION-1 <= current version $VERSION; aborting!" >&2
 	exit 1
     fi
 
     if [ -e "../$PACKAGE-$SNEW_VERSION" ]; then
-	echo "$PACKAGE-$SNEW_VERSION already exists in the parent directory!" >&2
+	echo "$PROGNAME: $PACKAGE-$SNEW_VERSION already exists in the parent directory!" >&2
 	echo "Aborting...." >&2
 	exit 1
     fi
     if [ -e "../$PACKAGE-$SNEW_VERSION.orig" ]; then
-	echo "$PACKAGE-$SNEW_VERSION.orig already exists in the parent directory!" >&2
+	echo "$PROGNAME: $PACKAGE-$SNEW_VERSION.orig already exists in the parent directory!" >&2
 	echo "Aborting...." >&2
 	exit 1
     fi
     if [ ! -r "../${PACKAGE}_$UVERSION.orig.tar.gz" ]; then
-	echo "Can't find/read ${PACKAGE}_$UVERSION.orig.tar.gz in the parent directory!" >&2
+	echo "$PROGNAME: can't find/read ${PACKAGE}_$UVERSION.orig.tar.gz" >&2
+	echo "in the parent directory!" >&2
 	echo "Aborting...." >&2
 	exit 1
     fi
@@ -260,12 +343,14 @@ if [ "$PATCH" ]; then
     # Clean package
     if [ -n "$UUPDATE_ROOTCMD" ]; then
 	debuild -r"$UUPDATE_ROOTCMD" clean || {
-	    echo "Couldn't run debuild -r$UUPDATE_ROOTCMD clean successfully.  Why not?" >&2
+	    echo "$PROGNAME: couldn't run  debuild -r$UUPDATE_ROOTCMD clean" >&2
+	    echo "successfully.  Why not?" >&2
 	    echo "Aborting...." >&2
 	    exit 1
 	}
     else debuild clean || {
-	    echo "Couldn't run debuild -r$UUPDATE_ROOTCMD clean successfully.  Why not?" >&2
+	    echo "$PROGNAME: couldn't run  debuild -r$UUPDATE_ROOTCMD clean" >&2
+	    echo "successfully.  Why not?" >&2
 	    echo "Aborting...." >&2
 	    exit 1
 	}
@@ -273,8 +358,34 @@ if [ "$PATCH" ]; then
 
     cd ..
     rm -rf $PACKAGE-$UVERSION.orig
-    tar zxf ${PACKAGE}_$UVERSION.orig.tar.gz
+
+    # Unpacking .orig.tar.gz is not quite trivial any longer ;-)
+    mkdir $TEMP_DIR || {
+	echo "$PROGNAME: can't create temporary directory `pwd`/$TEMP_DIR;" >&2
+	echo "aborting..." >&2
+	exit 1
+    }
+    cd $TEMP_DIR
+    tar zxf ../${PACKAGE}_$UVERSION.orig.tar.gz || {
+	echo "$PROGNAME: can't untar ${PACKAGE}_$UVERSION.orig.tar.gz;" >&2
+	echo "aborting..." >&2
+	exit 1
+    }
+    if [ `ls | wc -l` -eq 1 ] && [ -d "`ls`" ]; then
+	mv "`ls`" ../${PACKAGE}-$UVERSION.orig
+    else
+	mkdir ../$PACKAGE-$UVERSION.orig
+	mv * ../$PACKAGE-$UVERSION.orig
+    fi
+    cd ..
+    rm -rf $TEMP_DIR
+
     cd $PACKAGE-$UVERSION.orig
+    if ! $CATPATCH > /dev/null; then
+	echo "$PROGNAME: can't run $CATPATCH;" >&2
+	echo "aborting..." >&2
+	exit 1
+    fi
     if $CATPATCH | patch -sp1; then
 	cd ..
 	mv $PACKAGE-$UVERSION.orig $PACKAGE-$SNEW_VERSION.orig
@@ -284,7 +395,7 @@ if [ "$PATCH" ]; then
 	if $CATPATCH | patch -sp1; then
 	    echo "Success. The supplied diffs worked fine on the Debian sources."
 	else
-	    echo "The diffs from did not apply cleanly!" >&2
+	    echo "$PROGNAME: the diffs supplied did not apply cleanly!" >&2
 	    X="`find . -name "*.rej"`"
 	    echo "Rejected diffs are in $X" >&2
 	    STATUS=1
@@ -295,7 +406,7 @@ if [ "$PATCH" ]; then
 	echo "Do a \"cd ../$PACKAGE-$SNEW_VERSION\" to see the new package"
 	exit
     else
-	echo "Patch failed to apply to original sources $UVERSION" >&2
+	echo "$PROGNAME: patch failed to apply to original sources $UVERSION" >&2
 	cd ..
 	rm -rf $PACKAGE-$UVERSION.orig
 	exit 1
@@ -303,13 +414,13 @@ if [ "$PATCH" ]; then
 else
 # This is an orginal sourcearchive
     if [ "$ARCHIVE" = "" ]; then
-	echo "Upstream source archive not specified" >&2
+	echo "$PROGNAME: upstream source archive not specified" >&2
 	exit 1
     fi
     case "$ARCHIVE" in
 	/*)
 	    if [ ! -r "$ARCHIVE" ]; then
-		echo "Cannot read archive file $ARCHIVE!  Aborting." >&2
+		echo "$PROGNAME: cannot read archive file $ARCHIVE!  Aborting." >&2
 		exit 1
 	    fi
 	    ARCHIVE_PATH="$ARCHIVE"
@@ -322,7 +433,7 @@ else
 	    elif [ -r "$OPWD/$ARCHIVE" ]; then
 		ARCHIVE_PATH="$OPWD/$ARCHIVE"
 	    else
-		echo "Cannot read archive file $ARCHIVE!  Aborting." >&2
+		echo "$PROGNAME: cannot read archive file $ARCHIVE!  Aborting." >&2
 		exit 1
 	    fi
 	    
@@ -332,6 +443,7 @@ else
     # Figure out the type of archive
     X="${ARCHIVE##*/}"
     case "$X" in
+	*.orig.tar.gz)  X="${X%.orig.tar.gz}";  UNPACK="tar zxf" ;;
 	*.tar.gz)  X="${X%.tar.gz}";  UNPACK="tar zxf" ;;
 	*.tar.bz2) X="${X%.tar.bz2}"; UNPACK="tar --bzip2 -xf" ;;
 	*.tar.Z)   X="${X%.tar.Z}";   UNPACK="tar zxf" ;;
@@ -339,15 +451,15 @@ else
 	*.tar)     X="${X%.tar}";     UNPACK="tar xf"  ;;
 	*.zip)     X="${X%.zip}";     UNPACK="unzip"   ;;
 	*)
-	    echo "Sorry: Unknown archive type" >&2
+	    echo "$PROGNAME: sorry: Unknown archive type" >&2
 	    exit 1
     esac
     if [ "$NEW_VERSION" = "" ]; then
 	# Figure out the new version
 	NEW_VERSION=`echo "$X" | perl -ne "/$MPATTERN/"' && print $1'`
 	if [ -z "$NEW_VERSION" ]; then
-	    echo "New version number not recognized from given filename" >&2
-	    echo "Please run uupdate with the -v option" >&2
+	    echo "$PROGNAME: new version number not recognized from given filename" >&2
+	    echo "Please run $PROGNAME with the -v option" >&2
 	    exit 1
 	fi
     fi
@@ -365,17 +477,17 @@ else
 
     # Sanity check
     if dpkg --compare-versions "$NEW_VERSION-1" le "$VERSION"; then
-	echo "New version $NEW_VERSION-1 <= current version $VERSION; aborting!" >&2
+	echo "$PROGNAME: new version $NEW_VERSION-1 <= current version $VERSION; aborting!" >&2
 	exit 1
     fi
 
     if [ -e "../$PACKAGE-$SNEW_VERSION.orig" ]; then
-	echo "Original source tree already exists as $PACKAGE-$SNEW_VERSION.orig!" >&2
+	echo "$PROGNAME: original source tree already exists as $PACKAGE-$SNEW_VERSION.orig!" >&2
 	echo "Aborting...." >&2
 	exit 1
     fi
     if [ -e "../$PACKAGE-$SNEW_VERSION" ]; then
-	echo "Source tree for new version already exists as $PACKAGE-$SNEW_VERSION!" >&2
+	echo "$PROGNAME: source tree for new version already exists as $PACKAGE-$SNEW_VERSION!" >&2
 	echo "Aborting...." >&2
 	exit 1
     fi
@@ -385,34 +497,55 @@ else
 	[ `md5sum "${ARCHIVE_PATH}" | cut -d" " -f1` != \
 	  `md5sum "../${PACKAGE}_$SNEW_VERSION.orig.tar.gz" | cut -d" " -f1` ]
     then
-	echo "A different ${PACKAGE}_$SNEW_VERSION.orig.tar.gz already exists in the parent dir;" >&2
-	echo "  please check on the situation before trying uupdate again." >&2
+	echo "$PROGNAME: a different ${PACKAGE}_$SNEW_VERSION.orig.tar.gz" >&2
+	echo "already exists in the parent dir;" >&2
+	echo " please check on the situation before trying $PROGNAME again." >&2
 	exit 1
     fi
 
     if [ $UUPDATE_PRISTINE = yes -a \
 	! -e "../${PACKAGE}_$SNEW_VERSION.orig.tar.gz" ]; then
-	echo "Symlinking to pristine source from ${PACKAGE}_$SNEW_VERSION.orig.tar.gz..."
+	if [ "$UUPDATE_SYMLINK_ORIG" = yes ]; then
+	    echo "Symlinking to pristine source from ${PACKAGE}_$SNEW_VERSION.orig.tar.gz..."
+	else
+	    echo "Copying pristine source to ${PACKAGE}_$SNEW_VERSION.orig.tar.gz..."
+	fi
 	case $ARCHIVE_PATH in
 	    /*.tar.gz|/*.tgz)
-		ln -s "$ARCHIVE" "../${PACKAGE}_$SNEW_VERSION.orig.tar.gz"
+		if [ "$UUPDATE_SYMLINK_ORIG" = yes ]; then
+		    ln -s "$ARCHIVE" "../${PACKAGE}_$SNEW_VERSION.orig.tar.gz"
+		else
+		    cp "$ARCHIVE" "../${PACKAGE}_$SNEW_VERSION.orig.tar.gz"
+		fi
 		;;
 	    ../*.tar.gz|../*.tgz)
-		ln -s "${ARCHIVE#../}" "../${PACKAGE}_$SNEW_VERSION.orig.tar.gz"
+		if [ "$UUPDATE_SYMLINK_ORIG" = yes ]; then
+		    ln -s "${ARCHIVE#../}" "../${PACKAGE}_$SNEW_VERSION.orig.tar.gz"
+		else
+		    cp "${ARCHIVE#../}" "../${PACKAGE}_$SNEW_VERSION.orig.tar.gz"
+		fi
 		;;
 		# no other ARCHIVE_PATH possibilities: either absolute or ../
 	    *)
-		echo "Can't preserve pristine sources from non .tar.gz upstream archive!" >&2
+		echo "$PROGNAME: can't preserve pristine sources from non .tar.gz upstream archive!" >&2
 		echo "Continuing anyway..." >&2
 		;;
 	esac
     fi
 
     cd ..
-    mkdir $TEMP_DIR || exit 1
+    mkdir $TEMP_DIR || {
+	echo "$PROGNAME: can't create temporary directory `pwd`/$TEMP_DIR;" >&2
+	echo "aborting..." >&2
+	exit 1
+    }
     cd $TEMP_DIR
     echo "-- Untarring the new sourcecode archive $ARCHIVE"
-    $UNPACK "$ARCHIVE_PATH"
+    $UNPACK "$ARCHIVE_PATH" || {
+	echo "$PROGNAME: can't unpack: $UNPACK $ARCHIVE_PATH failed;" >&2
+	echo "aborting..." >&2
+	exit 1
+    }
     cd ..
     if [ `ls $TEMP_DIR | wc -l` -eq 1 ]; then
 	# The files are stored in the archive under a top directory, we presume
@@ -425,17 +558,76 @@ else
     rm -rf $TEMP_DIR
     cp -a $PACKAGE-$SNEW_VERSION $PACKAGE-$SNEW_VERSION.orig
     cd $PACKAGE-$SNEW_VERSION
-    if [ -r "../${PACKAGE}_$SVERSION.diff.gz" ] &&
-	    zcat ../${PACKAGE}_$SVERSION.diff.gz | patch -sNp1 ; then
-	echo "Success!  The diffs from version $VERSION worked fine."
-    elif [ ! -r "../${PACKAGE}_$SVERSION.diff.gz" ]; then
-	echo "Could not find diffs from version $VERSION to apply!" >&2
-	exit 1
+    if [ -r "../${PACKAGE}_$SVERSION.diff.gz" ];
+    then
+	# Check that any files added in diff do not now exist in
+	# upstream version
+	FILES=$(zcat ../${PACKAGE}_$SVERSION.diff.gz |
+	        perl -nwe 'BEGIN { $status=""; }
+	                   chomp;
+	                   if (/^--- /) { $status = "-$."; }
+	                   if (/^\+\+\+ (.*)/ and $status eq ("-" . ($.-1))) {
+	                       $file = $1;
+	                       $file =~ s%^[^/]+/%%;
+	                       $status = "+$.";
+	                   }
+	                   if (/^@@ -([^ ]+) /) {
+	                       if ($1 eq "0,0" and $status eq ("+" . ($.-1))) {
+	                           print "$file\n";
+	                       }
+	                   }')
+
+	# Note that debian/changelog is always in FILES, so FILES is non-null
+	# Check anyway!
+	if [ -z "$FILES" ]; then
+	    echo "$PROGNAME: FILES is empty - why?  Please file a bug report!" >&2
+	    echo "Aborting..." >&2
+	    exit 1
+	fi
+
+	for file in $FILES; do
+	    if [ -e "$file" ]; then
+		echo "$PROGNAME warning: file $file was added in old diff, but is now in the upstream source." >&2
+		echo "Please check that the diff is applied correctly." >&2
+		echo "(This program will use the pristine upstream version and save the old .diff.gz" >&2
+		echo "version as $file.debdiff .)" >&2
+
+		if [ -e "$file.upstream" -o -e "$file.debdiff" ]; then
+		    FILEEXISTERR=1
+		fi
+	    fi
+	done
+
+	if [ -n "$FILEEXISTERR" ]; then
+	    echo "$PROGNAME: please apply the diff by hand and take care with this." >&2
+	    exit 1
+	fi
+
+	for file in $FILES; do
+	    if [ -e "$file" ]; then
+		mv $file $file.upstream
+	    fi
+	done
+
+	if zcat ../${PACKAGE}_$SVERSION.diff.gz | patch -sNp1 ; then
+	    echo "Success!  The diffs from version $VERSION worked fine."
+	else
+	    echo "$PROGNAME: the diffs from version $VERSION did not apply cleanly!" >&2
+	    X=$(find . -name "*.rej")
+	    echo "Rejected diffs are in $X" >&2
+	    STATUS=1
+	fi
+
+	for file in $FILES; do
+	    if [ -e "$file.upstream" ]; then
+		mv $file $file.debdiff
+		mv $file.upstream $file
+	    fi
+	done
+
     else
-	echo "The diffs from version $VERSION did not apply cleanly!" >&2
-	X="`find . -name "*.rej"`"
-	echo "Rejected diffs are in $X" >&2
-	STATUS=1
+	echo "$PROGNAME: could not find diffs from version $VERSION to apply!" >&2
+	exit 1
     fi
     chmod a+x debian/rules
     debchange -v "$NEW_VERSION-1" New upstream release
